import 'package:flutter/material.dart';
import 'dart:math' as Math;

class AddCustomWidget extends StatefulWidget {
  final double height;
  final double width;
  final int dotNumber; //小圆点的个数
  final bool isExe;
  final Color color;

  const AddCustomWidget(
      {Key? key,
      this.color = Colors.white,
      this.height = 200,
      this.width = 200,
      this.isExe = false,
      this.dotNumber = 60})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _AddCustomState();
  }
}

class _AddCustomState extends State<AddCustomWidget>
    with TickerProviderStateMixin {
  AnimationController? _controller;
  Animation<double>? _lineAnimation;

  //是否正在执行
  @override
  void initState() {
    super.initState();
    //旋转动画控制器
    _controller =
        AnimationController(duration: const Duration(seconds: 240), vsync: this)
          ..addListener(() {
            setState(() {});
          });
    _lineAnimation = Tween(begin: 0.0, end: 7200.0).animate(
        CurvedAnimation(curve: const Interval(0.0, 1), parent: _controller!));
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.isExe && _controller?.status.name != "forward") {
      _controller?.forward();
    } else if (!widget.isExe) {
      _controller?.reset();
    }

    return Center(
        child: CustomPaint(
      painter:
          _CustomWidget(index: _lineAnimation!.value, number: widget.dotNumber,color: widget.color),
      size: Size(widget.width, widget.height),
    ));
  }
}

class _CustomWidget extends CustomPainter {
  int number; //总个数
  double index; //当前值
  final Color color;

  _CustomWidget({this.number = 60, this.index = 0, required this.color});

  final double minCircleR = 1.5; //小圆的半径
  final double maxCircleR = 4; //大圆点的半径

  @override
  void paint(Canvas canvas, Size size) {
    var rect = Offset.zero & size;
    _canvasCircleDot(rect, canvas, size);
    _canvasCircle(rect, canvas, size);
  }

  ///绘制圆圈
  _canvasCircle(Rect rect, Canvas canvas, Size size) {
    var paint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.stroke
      ..color = color;
    double r = rect.width / 2;
    paint.style = PaintingStyle.fill; //填充
    double minCircle = 360 / number; //每一个圆间隔的角度度
    double hudu; //每一个小圆直接的弧度
    Offset offset;
    hudu = (2 * Math.pi / 360) * minCircle * index;
    offset = Offset(rect.center.dx + (r * Math.cos(hudu)),
        rect.center.dy + r * Math.sin(hudu));
    canvas.drawCircle(offset, maxCircleR, paint);
  }

  ///绘制圆点
  _canvasCircleDot(Rect rect, Canvas canvas, Size size) {
    var paint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.stroke
      ..color = color;
    double r = rect.width / 2;
    paint.style = PaintingStyle.fill; //填充
    double minCircle = 360 / number; //每一个圆间隔的角度度
    double hudu; //每一个小圆直接的弧度
    Offset offset;
    for (int i = 0; i < number; i++) {
      hudu = (2 * Math.pi / 360) * minCircle * i;
      offset = Offset(rect.center.dx + (r * Math.cos(hudu)),
          rect.center.dy + r * Math.sin(hudu));
      canvas.drawCircle(offset, minCircleR, paint);
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
