import 'package:flutter/material.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/widget/HomeListData.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///更多
class MoreOptionsWidget extends StatefulWidget {
  final List<HomeListData> list;

  final String deviceId;

  const MoreOptionsWidget(
      {Key? key, required this.deviceId, required this.list})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return MoreOptionsState();
  }
}

class MoreOptionsState extends State<MoreOptionsWidget> {
  String deviceName = "";

  @override
  void initState() {
    super.initState();
    _getDeviceName();
  }

  ///获得设备名称
  _getDeviceName() async {
    var map = await DeviceManage().device(widget.deviceId);
    deviceName = (map != null) ? map["deviceName"] : "";
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
            title: Text(S.of(context).device_moreOptions + "($deviceName)")),
        body: Container(
            margin: const EdgeInsets.fromLTRB(0, 10, 0, 10),
            child: GridView.builder(
                gridDelegate:  SliverGridDelegateWithFixedCrossAxisCount(
                    crossAxisSpacing: 10,
                    mainAxisSpacing: 10,
                    crossAxisCount:  (MediaQuery.of(context).size.width ~/ 100),
                    childAspectRatio: 1.2),
                itemCount: widget.list.length,
                itemBuilder: (context, index) {
                  return GestureDetector(
                      onTap: () {
                        if (widget.list[index].widget == null) return;
                        Navigator.of(context)
                            .push(MaterialPageRoute(builder: (context) {
                          return widget.list[index].widget!;
                        })).then((value) {
                          _getDeviceName();
                        });
                      },
                      child: Container(
                        margin: const EdgeInsets.fromLTRB(10, 0, 10, 0),
                        decoration: BoxDecoration(
                          border: Border.all(
                              color: Theme.of(context).primaryColorLight),
                          borderRadius:
                              const BorderRadius.all(Radius.circular(4.0)),
                        ),
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                          children: [
                            Icon(
                              widget.list[index].icon,
                              size: 30,
                              color: Theme.of(context).primaryColorLight,
                            ),
                            Text(
                              widget.list[index].name,
                              style: TextStyle(
                                  color: Theme.of(context).primaryColorLight,
                                  fontSize: 12),
                            )
                          ],
                        ),
                      ));
                })));
  }
}
