import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/config/WitsystemConfig.dart';
import 'package:witsystem_core/file/WitsFile.dart';
import 'package:witsystem_core/pages/code/WitsQrIamge.dart';
import 'package:witsystem_core/permission/WitsPermission.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///设备二维码

class DeviceQrcodeWidget extends StatefulWidget {
  final String deviceId;

  const DeviceQrcodeWidget({Key? key, required this.deviceId})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _DeviceQrcodeState();
  }
}

class _DeviceQrcodeState extends State<DeviceQrcodeWidget> {
  String action = "unlock";
  String deviceName = "";

  @override
  void initState() {
    super.initState();
    _getDeviceName();
  }

  ///选择的功能
  _selectItem(value) {
    setState(() {
      action = value;
    });
  }

  ///保存二维码
  _saveQrcode() async {
    if (!await WitsPermission.photosPermission()) {
      EasyLoading.showToast(S.current.device_missingPhotoPermissions);
      return;
    }
    EasyLoading.show(status: S.current.device_saveIn);
    var saveNetImage = await WitsFile.saveNetImage(
        url:
            "/file/created_qrCode_file?qrCodeData=${WitsystemConfig().deviceQrcodeUrl(deviceId: widget.deviceId, action: action)}",
        fileName: deviceName + "-" + action + ".jpg");
    EasyLoading.dismiss();
    debugPrint("保存图片$saveNetImage");
    if (saveNetImage["isSuccess"]) {
      EasyLoading.showToast(S.current.device_saveSuccess);
    } else {
      EasyLoading.showToast(
          S.current.device_saveFail + "${saveNetImage["errorMessage"]}");
    }
  }

  ///获得设备名称
  _getDeviceName() async {
    deviceName = (await DeviceManage().device(widget.deviceId))!["deviceName"];
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(S.of(context).device_deviceQrCode + "($deviceName)"),
      ),
      body: ListView(children: [
        Padding(
            padding: const EdgeInsets.fromLTRB(10, 30, 10, 30),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                GestureDetector(
                  onTap: _saveQrcode,
                  child: WitsQrIamge(
                    data: WitsystemConfig().deviceQrcodeUrl(
                        deviceId: widget.deviceId, action: action),
                    size: 240.0,
                  ),
                ),
              ],
            )),
        const Divider(height: 1),
        Padding(
            padding: const EdgeInsets.all(20),
            child: Column(children: [
              Text(S.of(context).device_selectScanFunction),
              Text(S.of(context).device_clickSaveImage),
            ])),

        Row(
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            MaterialButton(
              onPressed: () {
                _selectItem("unlock");
              },
              color: action == "unlock"
                  ? Theme.of(context).primaryColorLight
                  : Colors.white,
              padding: const EdgeInsets.fromLTRB(20, 5, 20, 5),
              shape: RoundedRectangleBorder(
                side: BorderSide(
                  color: Theme.of(context).primaryColorLight,
                  width: 1,
                ),
                borderRadius: const BorderRadius.all(
                  Radius.circular(8),
                ),
              ),
              child: Text(S.of(context).device_openDevice,
                  style: TextStyle(
                      fontSize: 14,
                      color: action == "unlock" ? Colors.white : Colors.black)),
            ),
            MaterialButton(
              onPressed: () {
                _selectItem("addDevice");
              },
              color: action == "addDevice"
                  ? Theme.of(context).primaryColorLight
                  : Colors.white,
              padding: const EdgeInsets.fromLTRB(20, 5, 20, 5),
              shape: RoundedRectangleBorder(
                side: BorderSide(
                  color: Theme.of(context).primaryColorLight,
                  width: 1,
                ),
                borderRadius: const BorderRadius.all(
                  Radius.circular(8),
                ),
              ),
              child: Text(S.of(context).device_addDevice,
                  style: TextStyle(
                      fontSize: 14,
                      color:
                          action == "addDevice" ? Colors.white : Colors.black)),
            ),
            MaterialButton(
              onPressed: () {
                _selectItem("unlockOrAddDevice");
              },
              color: action == "unlockOrAddDevice"
                  ? Theme.of(context).primaryColorLight
                  : Colors.white,
              shape: RoundedRectangleBorder(
                side: BorderSide(
                  color: Theme.of(context).primaryColorLight,
                  width: 1,
                ),
                borderRadius: const BorderRadius.all(
                  Radius.circular(8),
                ),
              ),
              child: Text(S.of(context).device_addOrOpen,
                  style: TextStyle(
                      fontSize: 14,
                      color: action == "unlockOrAddDevice"
                          ? Colors.white
                          : Colors.black)),
            ),
          ],
        )

        //SelectGroup
      ]),
    );
  }
}
