import 'package:date_time_picker/date_time_picker.dart';
import 'package:flutter/material.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/share/ShortShareInfo.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///设备当面分享页面
///TextField 介绍 https://www.jianshu.com/p/421f0812cbe8

class DeviceShortShareWidget extends StatefulWidget {
  final String title;

  final String deviceId;

  const DeviceShortShareWidget({
    Key? key,
    required this.title,
    required this.deviceId,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _DeviceShortShareState();
  }
}

class _DeviceShortShareState extends State<DeviceShortShareWidget> {
  String deviceName = "";
  final _authType = TextEditingController();
  final _startTime = TextEditingController();
  final _endTime = TextEditingController();
  final List<Map<String, dynamic>> _authSheetItem = [
    {"text": S.current.device_admin, "auth": "Admin"},
    {"text": S.current.device_common, "auth": "Common"},
    //{"text": S.current.device_cycle, "auth": "Cycle"},//周期权限
    {"text": S.current.device_disposable, "auth": "Disposable"},
    //{"text": S.current.device_wxDisposable, "auth": "Look"}, //查看权限
    //{"text": S.current.device_cancel, "auth": "WX_Disposable"},//微信一次性钥匙暂时不实现
    {"text": S.current.device_cancel, "auth": ""},
  ]; // 0 代表管理员 1代表普通用户 2代表周期用户 3代表一次性用户 4代表只有查看权限

  ///选择权限
  _selectAuth() {
    Future.delayed(const Duration(milliseconds: 160), () {
      WitsystemBottomSheet().show(
        context: context,
        sheetItem: _authSheetItem,
        showKey: "text",
        onTap: (index) {
          if (index == _authSheetItem.length - 1) {
            _authType.text = "";
          } else {
            _authType.text = _authSheetItem[index]["text"];
          }
        },
      );
    });
  }

  ///给开始时间添加动画和判断是否显示
  Widget _widgetStartTimeAnimation({required Widget child}) {
    return AnimatedSwitcher(
      duration: const Duration(milliseconds: 200),
      transitionBuilder: (Widget child, Animation<double> animation) {
        return ScaleTransition(child: child, scale: animation); //进行自定义的切换动画
      },
      child: (_authType.value.text != "" &&
              _authType.value.text != _authSheetItem[0]["text"])
          ? child
          : const SizedBox(height: 0, width: 0),
    );
  }

  ///给结束时间添加动画和判断是否显示
  Widget _widgetEndTimeAnimation({required Widget child}) {
    return AnimatedSwitcher(
      duration: const Duration(milliseconds: 200),
      transitionBuilder: (Widget child, Animation<double> animation) {
        return ScaleTransition(child: child, scale: animation); //进行自定义的切换动画
      },
      child: (_authType.value.text != "" &&
              _authType.value.text != _authSheetItem[0]["text"] &&
              _startTime.value.text != "")
          ? child
          : const SizedBox(height: 0, width: 0),
    );
  }

  ///给结束时间添加动画和判断是否显示
  Widget _widgetButtonAnimation({required Widget child}) {
    return AnimatedSwitcher(
      duration: const Duration(milliseconds: 200),
      transitionBuilder: (Widget child, Animation<double> animation) {
        return ScaleTransition(child: child, scale: animation); //进行自定义的切换动画
      },
      child: ((_authType.value.text != "" &&
                  _authType.value.text == _authSheetItem[0]["text"]) ||
              (_authType.value.text != "" &&
                  _startTime.value.text != "" &&
                  _endTime.value.text != ""))
          ? child
          : const SizedBox(height: 0, width: 0),
    );
  }

  ///分享按键
  _shareButton() {
    _createQrCode();
  }

  ///去生成二维码 当面分享
  _createQrCode() {
    Navigator.of(context).push(MaterialPageRoute(builder: (context) {
      return ShortShareInfoWidget(
        deviceName: deviceName,
        deviceId: widget.deviceId,
        startTime: _startTime.text.isEmpty ? null : _startTime.text + ":00",
        endTime: _endTime.text.isEmpty ? null : _endTime.text + ":00",
        authType: __authTypeValueToInt(),
        repeatType: "Day",
      );
    }));
  }

  ///清空选项
  _clean() {
    _authType.text = "";
    _startTime.text = "";
    _endTime.text = "";
    setState(() {});
  }

  ///_authType 的值读取权限标记
  String __authTypeValueToInt() {
    var text = _authType.value.text;
    for (var value in _authSheetItem) {
      if (value["text"] == text) return value["auth"];
    }
    return "";
  }

  @override
  void initState() {
    super.initState();
    _getDeviceName();
  }

  ///获得设备名称
  _getDeviceName() async {
    deviceName = (await DeviceManage().device(widget.deviceId))!["deviceName"];
    setState(() {});
  }

  @override
  void dispose() {
    super.dispose();
    _authType.dispose();
    _startTime.dispose();
    _endTime.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
            title: Text(
          widget.title + "($deviceName)",
          style: const TextStyle(fontSize: 16),
        )),
        body: GestureDetector(
          onTap: () {
            FocusScope.of(context).requestFocus(FocusNode()); // 触摸收起键盘
          },
          child: Container(
            margin: const EdgeInsets.all(10),
            alignment: Alignment.center,
            child: ListView(
              children: [
                Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                  Container(
                    width: 120,
                    height: 120,
                    margin: const EdgeInsets.fromLTRB(0, 20, 0, 20),
                    alignment: Alignment.center,
                    decoration: BoxDecoration(
                        borderRadius:
                            const BorderRadius.all(Radius.circular(60)),
                        border: Border.all(
                          color: Theme.of(context).primaryColor,
                        )),
                    child: Icon(Icons.share,
                        size: 30, color: Theme.of(context).primaryColor),
                  ),
                ]),
                Container(
                  margin: const EdgeInsets.fromLTRB(0, 0, 0, 20),
                  child: TextField(
                    onTap: () {
                      FocusScope.of(context).requestFocus(FocusNode());
                      _selectAuth();
                    },
                    controller: _authType,
                    textAlign: TextAlign.end,
                    //隐藏光标
                    showCursor: false,
                    //禁止复制
                    enableInteractiveSelection: false,
                    style: const TextStyle(fontSize: 14),
                    decoration: InputDecoration(
                        border: InputBorder.none,
                        prefixIcon: const Icon(WitsIcon.yonghu, size: 20),
                        prefixText: S.of(context).device_authType,
                        prefixStyle: const TextStyle(
                            fontSize: 14, color: Colors.black45),
                        hintText: S.of(context).device_selectAuthType,
                        hintStyle: const TextStyle(fontSize: 10),
                        suffixIcon: const Icon(WitsIcon.xiala, size: 20),
                        enabledBorder: const OutlineInputBorder(
                            borderRadius: BorderRadius.all(Radius.circular(10)),
                            borderSide: BorderSide(color: Colors.black12))),
                  ),
                ),
                _widgetStartTimeAnimation(
                    child: Container(
                  margin: const EdgeInsets.fromLTRB(0, 0, 0, 20),
                  child: DateTimePicker(
                    dateMask: 'yyyy-MM-dd HH:mm:ss',
                    controller: _startTime,
                    textAlign: TextAlign.end,
                    firstDate: DateTime(2000),
                    lastDate: DateTime(2100),
                    type: DateTimePickerType.dateTime,
                    style: const TextStyle(fontSize: 14),
                    decoration: InputDecoration(
                        border: InputBorder.none,
                        prefixIcon:
                            const Icon(WitsIcon.kaishishijian, size: 20),
                        prefixText: S.of(context).device_startTime,
                        prefixStyle: const TextStyle(
                            fontSize: 14, color: Colors.black45),
                        hintText: S.of(context).device_selectStartTime,
                        hintStyle: const TextStyle(fontSize: 10),
                        suffixIcon: const Icon(WitsIcon.xiala, size: 20),
                        enabledBorder: const OutlineInputBorder(
                            borderRadius: BorderRadius.all(Radius.circular(10)),
                            borderSide: BorderSide(color: Colors.black12)),
                        focusedBorder: const OutlineInputBorder(
                            borderRadius: BorderRadius.all(Radius.circular(10)),
                            borderSide: BorderSide(color: Colors.black12))),
                  ),
                )),
                _widgetEndTimeAnimation(
                    child: Container(
                  margin: const EdgeInsets.fromLTRB(0, 0, 0, 20),
                  child: DateTimePicker(
                    dateMask: 'yyyy-MM-dd HH:mm:ss',
                    firstDate: DateTime.now(),
                    lastDate: DateTime(2100),
                    controller: _endTime,
                    textAlign: TextAlign.end,
                    type: DateTimePickerType.dateTime,
                    style: const TextStyle(fontSize: 14),
                    decoration: InputDecoration(
                        border: InputBorder.none,
                        prefixIcon:
                            const Icon(WitsIcon.jieshushijian, size: 20),
                        prefixText: S.of(context).device_endTime,
                        prefixStyle: const TextStyle(
                            fontSize: 14, color: Colors.black45),
                        hintText: S.of(context).device_selectEndTime,
                        hintStyle: const TextStyle(fontSize: 10),
                        suffixIcon: const Icon(WitsIcon.xiala, size: 20),
                        enabledBorder: const OutlineInputBorder(
                            borderRadius: BorderRadius.all(Radius.circular(10)),
                            borderSide: BorderSide(color: Colors.black12)),
                        focusedBorder: const OutlineInputBorder(
                            borderRadius: BorderRadius.all(Radius.circular(10)),
                            borderSide: BorderSide(color: Colors.black12))),
                  ),
                )),
                _widgetButtonAnimation(
                    child: MaterialButton(
                  onPressed: _shareButton,
                  child: Text(S.of(context).device_createQrcode,
                      style: const TextStyle(color: Colors.white)),
                  color: Colors.blue,
                  height: 60,
                  minWidth: double.infinity,
                ))
              ],
            ),
          ),
        ));
  }
}
