import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/config/VibrateConfig.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/permission/WitsPermission.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/bindRelay/BindRelay.dart';
import 'package:witsystem_device_manage/pages/home/makeOver/MakeOver.dart';
import 'package:witsystem_device_manage/pages/home/ota/BleOta.dart';
import 'package:witsystem_device_manage/pages/home/qrcode/DeviceQrcode.dart';
import 'package:witsystem_device_manage/pages/home/share/DeviceShare.dart';
import 'package:witsystem_device_manage/pages/home/share/DeviceShortShare.dart';
import 'package:witsystem_device_manage/pages/home/slock/card/CardList.dart';
import 'package:witsystem_device_manage/pages/home/slock/info/DeviceInfo.dart';
import 'package:witsystem_device_manage/pages/home/slock/mark/MarkList.dart';
import 'package:witsystem_device_manage/pages/home/slock/password/PasswordList.dart';
import 'package:witsystem_device_manage/pages/home/slock/record/LockRecord.dart';
import 'package:witsystem_device_manage/pages/home/slock/setup/SlockSetup.dart';
import 'package:witsystem_device_manage/pages/home/slock/widget/UnlockWidget.dart';
import 'package:witsystem_device_manage/pages/home/userManage/UserManageWidget.dart';
import 'package:witsystem_device_manage/pages/scan/ScanCode.dart';
import 'package:witsystem_device_manage/pages/widget/HomeListData.dart';
import 'package:witsystem_device_manage/pages/widget/HomeTemplet.dart';
import 'package:witsystem_device_plugin/code/DeviceFunctionType.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///门锁主页
class SlcokHome extends StatefulWidget {
  final String deviceId;
  final String deviceName;

  //进入页面是否立即开锁
  final bool immediatelyOpenLock;

  /// const SlcokHome({Key? key}) :super(key: key);
  const SlcokHome(
      {Key? key,
      required this.deviceId,
      required this.deviceName,
      this.immediatelyOpenLock = false})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return SlcokHomeState();
  }
}

class SlcokHomeState extends State<SlcokHome> {
  List<HomeListData> list = [];
  StreamSubscription? _bleStateListen;
  Timer? _timer;
  String _hint = S.current.device_disconnectHint; //提示
  bool _exeLock = false; //true 代表开锁 false 代表只连接
  String deviceName = ""; //这样写是为了修改名称时候及时更换
  bool _connectClick = false; //点击首页进行连接，true 正在执行连接和认证，false 代表其他地方 连接，用户连接不用通过连接状态更新ui,这样设计是防止连接成功的回调快与认证的回调

  @override
  void initState() {
    super.initState();
    deviceName = widget.deviceName;
    _exeLock = widget.immediatelyOpenLock;
    _initListAll();
  }

  ///初始化所有功能列表
  _initListAll() {
    list = [
      HomeListData(
          icon: WitsIcon.yonghu,
          name: S.current.device_userManage,
          widget: UserManageWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.jianpan,
          name: S.current.device_passwordManage,
          hardware: DeviceFunctionType.bk,
          widget: PasswordList(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.zhiwen,
          name: S.current.device_markManage,
          hardware: DeviceFunctionType.mark,
          widget: MarkListWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.menka,
          name: S.current.device_cardManage,
          hardware: DeviceFunctionType.card,
          widget: CardListWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.fenxiang,
          name: S.current.device_shareDevice,
          widget: DeviceShareWidget(
              title: S.current.device_shareDevice, deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.erweima,
          name: S.current.device_shortShare,
          widget: DeviceShortShareWidget(
              title: S.current.device_shortShare, deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.jilu,
          name: S.current.device_LockRecord,
          widget: LockRecordWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.zhuanrang,
          name: S.current.device_makeOverDevice,
          widget: MakeOverWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.shebeishezhi,
          name: S.current.device_deviceSettings,
          widget: SlockSetupWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.luyouqi,
          name: S.current.device_bindRelay,
          widget: BindRelayWidget(
            deviceId: widget.deviceId,
          )),
      HomeListData(
          icon: WitsIcon.shebei,
          name: S.current.device_deviceInfo,
          widget: DeviceInfoWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.gujianshengji,
          name: S.current.device_ota,
          hardware: DeviceFunctionType.ota,
          widget: BleOtaWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.erweima,
          name: S.current.device_deviceQrCode,
          widget: DeviceQrcodeWidget(deviceId: widget.deviceId)),
      HomeListData(
          icon: WitsIcon.saoma,
          name: S.current.device_scanningCode,
          widget: const ScanCodeWidget()),
    ];
  }

  ///开门
  _unlock(UnlockState unlockState) async {
    try {
      if (_connectClick) {
        return;
      }
      _connectClick = true;
      if (!await WitsPermission.blePermission()) return;
      if (_timer != null) _timer?.cancel();

      unlockState.connect(); //执行正在连接动画
      _updateHintText(S.current.device_connecting); //显示正在连接提示文字

      await BleDevice().connectAndOpen(widget.deviceId, _exeLock,
          isVibrate: VibrateConfig().unlockVibrate); //调用执行开门

      if (_exeLock) {
        //true 代表开门成功 false 代表连接成功
        unlockState.openSuccess(); //调用显示成功的动画
        _updateHintText(S.current.device_openLockSuccess(deviceName));
        _timer = Timer(const Duration(seconds: 6), () {
          unlockState.connectSuccess();
          _updateHintText(S.current.device_connected); //显示已连接提示文字
        }); //定时器等待门锁关闭
      } else {
        _exeLock = true; //标记下一次一定是开门
        unlockState.connectSuccess();
        _updateHintText(S.current.device_connected); //显示已连接提示文字
      }
      _connectClick = false;
    } on PlatformException catch (e) {
      debugPrint("错误信息$e");
      _connectClick = false;
      _timer = Timer(const Duration(seconds: 5), () {
        unlockState.init();
        _updateHintText(S.current.device_disconnectHint);
      });
      unlockState.openFail();
      WitsystemError? error = AutoErrorHandle.auto(e);
      _updateHintText(error.error);
      setState(() {});
    }

    _listenBleConnectState(unlockState); //调用连接状态监听
  }

  ///更新页面提示文字
  _updateHintText(String hint) {
    setState(() {
      _hint = hint;
    });
  }

  ///监听和设备的连接状态
  _listenBleConnectState(UnlockState unlockState) async {
    //判断已经监听无须重复监听，防止重复监听
    if (_bleStateListen != null) {
      return;
    }
    _bleStateListen = BleDevice().bleDeviceConnectState((value) {
      if (_timer != null) _timer?.cancel();
      // print("首页监听设备的连接状态发生变化》》》》》》》》》》》》》》》》》1：：${value["newVal"] == 2}");
      if (int.parse(value["newVal"].toString()) == 2) {
        if (_connectClick) return;
        unlockState.connectSuccess();
        _updateHintText(S.current.device_connected); //显示已连接提示文字
      } else {
        unlockState.init();
        _updateHintText(S.current.device_disconnectHint);
      }
    });
  }

  ///取消监听状态
  _cancelListenBleConnectState() {
    if (_bleStateListen != null) {
      BleDevice().cancelBleDeviceConnectState(subscription: _bleStateListen);
    }
    _bleStateListen = null;
  }

  ///当返回该页面进行信息对比如果有更新就行信息更新
  _deviceInfoChange() async {
    Map<String, dynamic>? deviceInfo =
        await DeviceManage().device(widget.deviceId);
    if (deviceInfo == null) {
      //判断如果获得数据为空直接返回上一页
      Navigator.of(context).pop();
      EasyLoading.showToast(S.current.device_lackDevicePermissions);
      return;
    }
    if (deviceName != deviceInfo["deviceName"]) {
      deviceName = deviceInfo["deviceName"];
      setState(() {});
    }
  }

  @override
  void dispose() {
    super.dispose();
    BleDevice().closeBleDevice(widget.deviceId);
    if (_timer != null) _timer?.cancel();
    _cancelListenBleConnectState();
  }

  @override
  Widget build(BuildContext context) {
    return HomeTempletWidget(
        list: list,
        deviceId: widget.deviceId,
        deviceName: deviceName,
        itemOnTap: (HomeListData item, int index) {
          if (item.widget == null) return;
          Navigator.of(context).push(MaterialPageRoute(builder: (context) {
            return item.widget!;
          })).then((value) async {
            ///判断如果设备被删除返回上一级
            _deviceInfoChange();
          });
        },
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          mainAxisSize: MainAxisSize.min,
          children: [
            UnlockWidget(
              width: 160,
              height: 160,
              onTop: _unlock,
              immediatelyExecute: true,
            ),
            Container(
              margin: const EdgeInsets.fromLTRB(0, 30, 0, 10),
              child: Text(_hint,
                  style: const TextStyle(color: Colors.white, fontSize: 12)),
            ),
          ],
        ));
  }
}
