import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterHintWidget.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';

///密码信息

class MarkInfoWidget extends StatefulWidget {
  final String deviceName;
  final String deviceId;
  final Map markInfo;

  const MarkInfoWidget(
      {Key? key,
      required this.deviceName,
      required this.deviceId,
      required this.markInfo})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _MarkInfoState();
  }
}

class _MarkInfoState extends State<MarkInfoWidget> {
  final _markName = TextEditingController();
  final _authType = TextEditingController();
  final _state = TextEditingController();
  final _uid = TextEditingController();
  bool _isUpdateList = false; //是否更新上一页列表

  Map freeze = {"text": S.current.device_freeze, "type": "freeze"};
  Map unfreeze = {"text": S.current.device_unfreeze, "type": "unfreeze"};
  final List<Map> _operateSheetItem = [
    {"text": S.current.device_delete, "type": "delete"},
    {},
    {"text": S.current.device_cancel, "type": "cancel"},
  ];

  @override
  void initState() {
    super.initState();
    debugPrint("信息${widget.markInfo}");
    _markName.text = widget.markInfo["name"];
    _authType.text = widget.markInfo["dataType"];
    _state.text = !widget.markInfo["freeze"]
        ? S.current.device_normal
        : S.current.device_freeze;
    _uid.text =
        int.parse(widget.markInfo["uid"], radix: 16).toString().padLeft(4, "0");
  }

  ///点击卡片
  _cardOnTap() {
    _operateSheetItem[1] = widget.markInfo["freeze"] ? unfreeze : freeze;
    WitsystemBottomSheet().show(
      context: context,
      sheetItem: _operateSheetItem,
      showKey: "text",
      onTap: (index) {
        switch (_operateSheetItem[index]["type"]) {
          case "delete":
            _delete();
            break;
          case "freeze":
          case "unfreeze":
            _freeze();
            break;
        }
      },
    );
  }

  ///删除信息
  _delete() async {
    DialogCenterHint().show(
      context: context,
      title: S.current.device_deleteMark,
      content: S.current.device_deleteMarkHint(widget.deviceName),
      outSideDismissible: true,
      affirmText: S.of(context).device_delete,
      affirmTextColor: Colors.red,
      onPressed: (bool value) {
        if (value) _affirmDelete();
      },
    );
  }

  ///确认删除
  _affirmDelete() async {
    EasyLoading.show(status: S.current.device_loading);
    try {
      await BleDevice().deleteMark(widget.deviceId, widget.markInfo["uid"]);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_deleteSuccess);
      Future.delayed(const Duration(seconds: 1), () {
        Navigator.pop(context, true);
      });
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
    }
  }

  ///冻结或解冻
  _freeze() {
    DialogCenterHint().show(
      context: context,
      title: widget.markInfo["freeze"]
          ? S.current.device_unfreezeMarkHintTitle
          : S.current.device_freezeMarkHintTitle,
      content: widget.markInfo["freeze"]
          ? S.current.device_unfreezeMarkHint(widget.markInfo["name"])
          : S.current.device_freezeMarkHint(widget.markInfo["name"]),
      outSideDismissible: true,
      affirmText: widget.markInfo["freeze"]
          ? S.of(context).device_unfreeze
          : S.of(context).device_freeze,
      affirmTextColor: Colors.red,
      onPressed: (bool value) {
        if (value) _affirmFreeze();
      },
    );
  }

  ///确认冻结或解冻
  _affirmFreeze() async {
    try {
      EasyLoading.show(status: S.current.device_loading);
      await BleDevice().freezeMark(
          widget.deviceId, widget.markInfo["uid"], !widget.markInfo["freeze"]);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      widget.markInfo["freeze"] = !widget.markInfo["freeze"];
      _state.text = !widget.markInfo["freeze"]
          ? S.current.device_normal
          : S.current.device_freeze;
      setState(() {});
      _isUpdateList = true;
    } on PlatformException catch (e) {
      debugPrint("$e");
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
    }
  }

  @override
  void dispose() {
    super.dispose();
    _authType.dispose();
    _markName.dispose();
    _state.dispose();
    _uid.dispose();
    EasyLoading.dismiss();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: () {
          Navigator.pop(context, _isUpdateList);
          return Future.value(false);
        },
        child: Scaffold(
            appBar: AppBar(
                title: Text(
                    S.of(context).device_pwdDetail + "(${widget.deviceName})")),
            body: Container(
                margin: const EdgeInsets.fromLTRB(10, 0, 10, 0),
                child: ListView(children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Container(
                        height: 120,
                        width: 120,
                        margin: const EdgeInsets.fromLTRB(0, 20, 0, 20),
                        decoration: BoxDecoration(
                            border: Border.all(
                                color: Theme.of(context).primaryColorLight),
                            borderRadius:
                                const BorderRadius.all(Radius.circular(60))),
                        child: Icon(WitsIcon.zhiwen,
                            size: 50,
                            color: Theme.of(context).primaryColorLight),
                      ),
                    ],
                  ),
                  TextField(
                    onChanged: (v) {},
                    controller: _markName,
                    textAlign: TextAlign.end,
                    enableInteractiveSelection: false,
                    style: const TextStyle(fontSize: 14),
                    decoration: InputDecoration(
                      enabled: false,
                      prefixText: S.of(context).device_name,
                      prefixIcon: const Icon(WitsIcon.gaojian),
                    ),
                  ),
                  TextField(
                    onChanged: (v) {},
                    controller: _authType,
                    textAlign: TextAlign.end,
                    enableInteractiveSelection: false,
                    style: const TextStyle(fontSize: 14),
                    decoration: InputDecoration(
                      enabled: false,
                      prefixText: S.of(context).device_type,
                      prefixIcon: const Icon(WitsIcon.leixing),
                    ),
                  ),
                  TextField(
                    onChanged: (v) {},
                    controller: _state,
                    textAlign: TextAlign.end,
                    enableInteractiveSelection: false,
                    style: TextStyle(
                        fontSize: 14,
                        color: widget.markInfo["freeze"]
                            ? Colors.red
                            : Colors.black),
                    decoration: InputDecoration(
                      enabled: false,
                      prefixText: S.of(context).device_state,
                      prefixIcon: const Icon(WitsIcon.suozi),
                    ),
                  ),
                  TextField(
                    onChanged: (v) {},
                    controller: _uid,
                    textAlign: TextAlign.end,
                    enableInteractiveSelection: false,
                    style: const TextStyle(fontSize: 14),
                    decoration: InputDecoration(
                      enabled: false,
                      prefixText: S.of(context).device_uid,
                      prefixIcon: const Icon(WitsIcon.gengduo),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.fromLTRB(0, 40, 0, 0),
                    child: MaterialButton(
                      onPressed: _cardOnTap,
                      child: Text(S.of(context).device_operate,
                          style: const TextStyle(color: Colors.white)),
                      color: Theme.of(context).primaryColorLight,
                      padding: const EdgeInsets.all(20),
                    ),
                  )
                ]))));
  }
}
