import 'package:flutter/material.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/slock/mark/AddMark.dart';
import 'package:witsystem_device_manage/pages/home/slock/mark/MarkInfo.dart';
import 'package:witsystem_device_manage/pages/widget/FloatingActionButtonScaffold.dart';
import 'package:witsystem_device_manage/util/DeviceInfoAuth.dart';
import 'package:witsystem_device_plugin/command/MarkType.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';
import 'package:witsystem_device_plugin/enity/DeviceSoftwareInfo.dart';

///指纹列表
class MarkListWidget extends StatefulWidget {
  final String deviceId;

  const MarkListWidget({Key? key, required this.deviceId}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _MarkListState();
  }
}

class _MarkListState extends State<MarkListWidget> {
  String deviceName = "";

  ///指纹所有类型
  final Map<MarkType, dynamic> _supportMarkType = {
    MarkType.ADMIN_MARK: S.current.device_adminMark,
    MarkType.USER_MARK: S.current.device_userMark,
  };

  List list = [];

  @override
  void initState() {
    super.initState();
    _filterSupportPasswordType();
    initData();
  }

  ///筛选设备支持哪些类型指纹
  _filterSupportPasswordType() async {
    DeviceSoftwareInfo? softwareInfo =
        await DeviceManage().getDeviceSoftwareFunction(widget.deviceId);
    _supportMarkType
        .removeWhere((key, value) => !softwareInfo!.markType.contains(key));
  }

  ///初始化数据
  initData() async {
    Map<String, dynamic>? device = await DeviceManage().device(widget.deviceId);
    deviceName = device!["deviceName"];
    list.clear();
    if (device["device"]["devicesSlock"] == null) {
      setState(() {});
      return;
    }
    List deviceList = device["device"]["devicesSlock"];
    int length = deviceList.length;
    for (int i = length - 1; i >= 0; i--) {
      if (deviceList[i]["type"] == "MARK") {
        deviceList[i]["dataType"] =
            DeviceInfoAuth.markAuthToString(deviceList[i]["dataType"]);
        list.add(deviceList[i]);
      }
    }
    setState(() {});
  }

  ///刷新
  refurbishDevice() async {
    await DeviceManage().refurbishDevice(widget.deviceId);
    await initData();
  }

  ///选择权限
  _selectAuth() {
    Future.delayed(const Duration(milliseconds: 160), () {
      WitsystemBottomSheet().show(
        context: context,
        sheetItem: _supportMarkType.values.toList()
          ..add(S.current.device_cancel),
        onTap: (index) {
          if (index == _supportMarkType.length) {
            return;
          }
          Navigator.of(context).push(MaterialPageRoute(builder: (context) {
            return AddMarkWidget(
                deviceName: deviceName,
                deviceId: widget.deviceId,
                markType: _supportMarkType.keys.toList()[index]);
          })).then((value) {
            if (value) initData();
          });
        },
      );
    });
  }

  @override
  Widget build(BuildContext context) {
    return FloatingActionButtonScaffold(
      title: S.of(context).device_markManage + "($deviceName)",
      floatingButtonOnTap: _selectAuth,
      crossFadeState:
          list.isEmpty ? CrossFadeState.showFirst : CrossFadeState.showSecond,
      centerHint: S.of(context).device_addMark,
      body: RefreshIndicator(
          onRefresh: () async {
            await refurbishDevice();
            return Future.value();
          },
          child: ListView.builder(
              itemCount: list.length,
              itemBuilder: (context, index) {
                return Card(
                    margin: const EdgeInsets.all(10),
                    shape: const RoundedRectangleBorder(
                      borderRadius: BorderRadius.all(Radius.circular(15)),
                    ),
                    elevation: 10,
                    child: InkWell(
                      borderRadius: const BorderRadius.all(Radius.circular(15)),
                      onTap: () {
                        Navigator.of(context)
                            .push(MaterialPageRoute(builder: (context) {
                          return MarkInfoWidget(
                              deviceName: deviceName,
                              deviceId: widget.deviceId,
                              markInfo: list[index]);
                        })).then((value) {
                          if (value) initData();
                        });
                      },
                      child: ListTile(
                        contentPadding: const EdgeInsets.all(10),
                        leading: const CircleAvatar(
                          child: Icon(Icons.fingerprint),
                        ),
                        title: Text(list[index]["name"]),
                        subtitle: Text(
                            list[index]["freeze"]
                                ? S.of(context).device_freeze
                                : S.of(context).device_normal,
                            style: TextStyle(
                                color: list[index]["freeze"]
                                    ? Colors.red
                                    : Colors.black)),
                        trailing: Flex(
                          mainAxisSize: MainAxisSize.min,
                          direction: Axis.horizontal,
                          children: [
                            Text(
                              list[index]["dataType"],
                              style: TextStyle(
                                  color: Theme.of(context).primaryColor),
                            ),
                            const Icon(Icons.keyboard_arrow_right)
                          ],
                        ),
                      ),
                    ));
              })),
    );
  }
}
