import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterHintWidget.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';

///密码信息

class PasswordInfoWidget extends StatefulWidget {
  final String deviceName;
  final String deviceId;
  final Map passwordInfo;

  const PasswordInfoWidget(
      {Key? key,
      required this.deviceName,
      required this.deviceId,
      required this.passwordInfo})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _PasswordInfoState();
  }
}

class _PasswordInfoState extends State<PasswordInfoWidget> {
  final _pwdName = TextEditingController();
  final _authType = TextEditingController();
  final _password = TextEditingController();
  final _state = TextEditingController();
  final _uid = TextEditingController();

  @override
  void initState() {
    super.initState();
    print("信息${widget.passwordInfo}");
    _pwdName.text = widget.passwordInfo["name"];
    _authType.text = widget.passwordInfo["dataType"];
    _password.text = widget.passwordInfo["data"];
    _state.text = !widget.passwordInfo["freeze"]
        ? S.current.device_normal
        : widget.passwordInfo["freeze"].toString();
    _uid.text = int.parse(widget.passwordInfo["uid"], radix: 16)
        .toString()
        .padLeft(4, "0");
  }

  ///删除信息
  _delete() async {
    DialogCenterHint().show(
      context: context,
      title: S.current.device_deletePassword,
      content: S.current.device_deletePasswordHint(widget.deviceName),
      outSideDismissible: true,
      affirmText: S.of(context).device_delete,
      affirmTextColor: Colors.red,
      onPressed: (bool value) {
        if (value) _affirmDelete();
      },
    );
  }

  ///确认删除
  _affirmDelete() async {
    EasyLoading.show(status: S.current.device_loading);
    try {
      await BleDevice()
          .deletePassword(widget.deviceId, widget.passwordInfo["uid"]);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_deleteSuccess);
      Future.delayed(const Duration(seconds: 1), () {
        Navigator.pop(context, true);
      });
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
    }
  }

  @override
  void dispose() {
    super.dispose();
    _authType.dispose();
    _pwdName.dispose();
    _password.dispose();
    _state.dispose();
    _uid.dispose();
    EasyLoading.dismiss();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
            title: Text(
                S.of(context).device_pwdDetail + "(${widget.deviceName})")),
        body: Container(
            margin: const EdgeInsets.fromLTRB(10, 0, 10, 0),
            child: ListView(children: [
              Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Container(
                    height: 120,
                    width: 120,
                    margin: const EdgeInsets.fromLTRB(0, 20, 0, 20),
                    decoration: BoxDecoration(
                        border: Border.all(
                            color: Theme.of(context).primaryColorLight),
                        borderRadius:
                            const BorderRadius.all(Radius.circular(60))),
                    child: Icon(WitsIcon.mensuo,
                        size: 50, color: Theme.of(context).primaryColorLight),
                  ),
                ],
              ),
              TextField(
                onChanged: (v) {},
                controller: _pwdName,
                textAlign: TextAlign.end,
                enableInteractiveSelection: false,
                style: const TextStyle(fontSize: 14),
                decoration: InputDecoration(
                  enabled: false,
                  prefixText: S.of(context).device_name,
                  prefixIcon: const Icon(WitsIcon.gaojian),
                ),
              ),
              TextField(
                onChanged: (v) {},
                controller: _authType,
                textAlign: TextAlign.end,
                enableInteractiveSelection: false,
                style: const TextStyle(fontSize: 14),
                decoration: InputDecoration(
                  enabled: false,
                  prefixText: S.of(context).device_type,
                  prefixIcon: const Icon(WitsIcon.leixing),
                ),
              ),
              TextField(
                onChanged: (v) {},
                controller: _password,
                textAlign: TextAlign.end,
                enableInteractiveSelection: false,
                style: const TextStyle(fontSize: 14),
                decoration: InputDecoration(
                  enabled: false,
                  prefixText: S.of(context).device_password,
                  prefixIcon: const Icon(WitsIcon.mima),
                ),
              ),
              TextField(
                onChanged: (v) {},
                controller: _state,
                textAlign: TextAlign.end,
                enableInteractiveSelection: false,
                style: const TextStyle(fontSize: 14),
                decoration: InputDecoration(
                  enabled: false,
                  prefixText: S.of(context).device_state,
                  prefixIcon: const Icon(WitsIcon.suozi),
                ),
              ),
              TextField(
                onChanged: (v) {},
                controller: _uid,
                textAlign: TextAlign.end,
                enableInteractiveSelection: false,
                style: const TextStyle(fontSize: 14),
                decoration: InputDecoration(
                  enabled: false,
                  prefixText: S.of(context).device_uid,
                  prefixIcon: const Icon(WitsIcon.gengduo),
                ),
              ),
              Padding(
                padding: const EdgeInsets.fromLTRB(0, 40, 0, 0),
                child: MaterialButton(
                  onPressed: _delete,
                  child: Text(S.of(context).device_delete,
                      style: const TextStyle(color: Colors.white)),
                  color: Colors.red,
                  padding: const EdgeInsets.all(20),
                ),
              )
            ])));
  }
}
