import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

class VolumeWidget extends StatefulWidget {
  final String deviceName;
  final String deviceId;

  const VolumeWidget(
      {Key? key, required this.deviceName, required this.deviceId})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _VolumeState();
  }
}

class _VolumeState extends State<VolumeWidget> {
  bool _isUpdateList = false; //上一页是否需要更新
  double volumeValue = 0;
  Map? _deviceInfo;

  @override
  void initState() {
    super.initState();
    DeviceManage().device(widget.deviceId).then((value) {
      _deviceInfo = value;
      volumeValue = double.parse(value!["device"]["volume"].toString());
      setState(() {});
    });
  }

  ///修改音量
  _updateVolume(v) async {
    EasyLoading.show(
        status: S.current.device_execution,
        maskType: EasyLoadingMaskType.clear);
    try {
      await BleDevice()
          .updateVolume(widget.deviceId, v.round());
      _isUpdateList = true;
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
      volumeValue = double.parse(_deviceInfo!["device"]["volume"].toString());
      setState(() {});
    }
  }



  @override
  void dispose() {
    super.dispose();
    EasyLoading.dismiss();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: () {
          Navigator.pop(context, _isUpdateList);
          return Future.value(false);
        },
        child: Scaffold(
          appBar: AppBar(
              title: Text(
                  S
                      .of(context)
                      .device_lockSpeech + "(${widget.deviceName})")),
          body: Container(
            height: 500,
            padding: const EdgeInsets.all(10),
            alignment: Alignment.center,
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                Text(
                  volumeValue == 0
                      ? S
                      .of(context)
                      .device_mute
                      : S
                      .of(context)
                      .device_volume + " $volumeValue",
                  style: TextStyle(
                      color: volumeValue == 0 ? Colors.red : Colors.black),
                ),
                Row(
                  children: [
                    const Icon(Icons.volume_off_sharp),
                    Expanded(
                      child: Slider(
                          value: volumeValue,
                          max: 7,
                          divisions: 7,
                          onChanged: (v) {
                            volumeValue = v;
                            setState(() {});
                          },
                          onChangeEnd: _updateVolume
                      ),
                    ),
                    const Icon(Icons.volume_up),
                  ],
                ),
              ],
            ),
          ),
        ));
  }
}
