import 'dart:typed_data';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'dart:math' as Math;

///添加指纹动画页面

class MarkWidget extends StatefulWidget {
  final double width;
  final double height;
  final double totalProgress; //总进度
  final double progress; //当前进度
  final Color progressColor; //进度颜色
  final Color progressBackground; //进度背景颜色

  const MarkWidget(
      {Key? key,
      required this.width,
      required this.height,
      this.totalProgress = 100,
      this.progress = 0,
      this.progressColor = Colors.red,
      this.progressBackground = Colors.white})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _MarkState();
  }
}

class _MarkState extends State<MarkWidget> with TickerProviderStateMixin {
  @override
  Widget build(BuildContext context) {
    return Center(
        child: CustomPaint(
      size: Size(widget.width, widget.height),
      painter: _Mark(
          totalProgress: widget.totalProgress<=0?1:widget.totalProgress,
          progressColor: widget.progressColor,
          progressBackground: widget.progressBackground,
          progress: widget.progress<=0?0:widget.progress),
    ));
  }
}

class _Mark extends CustomPainter {
  final double totalProgress; //当前进度
  final double progress; //当前进度
  final Color progressColor; //进度颜色
  final Color progressBackground; //进度背景颜色

  _Mark(
      {required this.totalProgress,
      required this.progress,
      required this.progressColor,
      required this.progressBackground});

  @override
  void paint(Canvas canvas, Size size) {
    var rect = Offset.zero & size;
    _drawArc(canvas, rect);
  }

  //绘制圆
  _drawCircle(Canvas canvas, Rect rect) {
    double r = rect.width / 2;
    var progressPaint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.stroke
      ..strokeWidth = 5
      ..color = Colors.red;
    var offset = Offset(rect.center.dx, rect.center.dy);
    canvas.drawCircle(offset, r, progressPaint);
  }

  //  绘制圆弧
  _drawArc(Canvas canvas, Rect rect) {
    double percentage = progress / totalProgress;
    var pi = Math.pi;
    var pi2 = 2 * pi;
    var progressPaint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.stroke
      ..strokeWidth = 5
      ..color = progressBackground;

    canvas.drawArc(rect, (pi2 * percentage), pi2 - (pi2 * percentage), false,
        progressPaint);
    progressPaint.color = progressColor;

    canvas.drawArc(rect, 0, pi2 * percentage, false, progressPaint);
  }

  //绘制6变形

  _drawRect(Canvas canvas, Rect rect) {
    var paint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.stroke
      ..strokeWidth = 3
      ..color = Colors.white;
    Path path = Path();
    Path path2 = Path();
    double r = rect.width / 2;
    int number = 10;
    double minCircle = 360 / number; //每一个圆间隔的角度度
    double hudu; //每一个小圆直接的弧度
    var r2 = r - 10;
    for (int i = 0; i <= number; i++) {
      hudu = (2 * Math.pi / 360) * minCircle * i;
      if (i == 0)
        path.moveTo(rect.center.dx + (r * Math.cos(hudu)),
            rect.center.dy + r * Math.sin(hudu) - 2.5);
      path.lineTo(rect.center.dx + (r * Math.cos(hudu)),
          rect.center.dy + r * Math.sin(hudu));

      // if(i==0)path2.moveTo(rect.center.dx + (r2 * Math.cos(hudu)), rect.center.dy + r2 * Math.sin(hudu)-2.5);
      // path2.lineTo(rect.center.dx + (r2 * Math.cos(hudu)), rect.center.dy + r2 * Math.sin(hudu));
      if (i == 0)
        path2.moveTo(rect.center.dx + (r2 * Math.cos(hudu)),
            rect.center.dy + r2 * Math.sin(hudu) - 2.5);
      path2.lineTo(rect.center.dx + (r2 * Math.cos(hudu)),
          rect.center.dy + r2 * Math.sin(hudu));
    }
    canvas.drawPath(path, paint);

    canvas.drawPath(path2, paint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
