import 'dart:math' as Math;

import 'package:flutter/material.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';

///开门组件

class UnlockWidget extends StatefulWidget {
  final double height;

  final double width;

  final bool immediatelyExecute; //是否立刻执行点击的回调onTop一次

  final void Function(UnlockState unlockState) onTop;

  const UnlockWidget(
      {Key? key,
      required this.height,
      required this.width,
      required this.onTop,
      this.immediatelyExecute = false})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return UnlockState();
  }
}

//,WidgetsBindingObserver
class UnlockState extends State<UnlockWidget> with TickerProviderStateMixin {
  AnimationController? _controller;
  Animation<double>? _lineAnimation;

  AnimationController? _changeController;
  Animation<double>? _changeAnimation;

  AnimationController? _heartbeatController;
  Animation<double>? _heartbeatAnimation;

  IconData icon = WitsIcon.duankai;
  Color iconColor = Colors.white;

  @override
  void initState() {
    super.initState();
    //旋转动画控制器
    _controller =
        AnimationController(duration: const Duration(seconds: 120), vsync: this)
          ..addListener(() {
            setState(() {});
          });
    _lineAnimation = Tween(begin: 0.0, end: 360.0).animate(CurvedAnimation(
        curve: const Interval(0.0026, 1), parent: _controller!));

    _changeController = AnimationController(
        duration: const Duration(milliseconds: 200), vsync: this)
      ..addListener(() {
        setState(() {});
      }); //过渡动画

    _changeAnimation = Tween(begin: 3.0, end: 0.0).animate(CurvedAnimation(
        curve: const Interval(0.000, 1), parent: _changeController!));

    _heartbeatController = AnimationController(
        duration: const Duration(milliseconds: 1000), vsync: this)
      ..addListener(() {
        setState(() {});
      })
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          _heartbeatController?.reverse();
        } else if (status == AnimationStatus.dismissed) {
          _heartbeatController?.forward();
        }
      }); //心跳动画

    _heartbeatAnimation = Tween(begin: 0.0, end: 5.0).animate(CurvedAnimation(
        curve: const Interval(0.000, 1), parent: _heartbeatController!));
  }

  //点击事件
  _onTop() {
    widget.onTop(this);
  }

  //连接设备
  connect() {
    iconColor = Colors.white;
    icon = WitsIcon.lanya;
    _stateAnimation();
  }

  //连接设备成功
  connectSuccess() {
    iconColor = Colors.white;
    icon = WitsIcon.shangsuo;
    _stop();
  }

  //开启设备成功
  openSuccess() {
    _stop();
    iconColor = Colors.white;
    icon = WitsIcon.kaisuo;
    _changeController?.forward();
  }

  //开启设备失败
  openFail() {
    _stop();
    icon = WitsIcon.shibai;
    iconColor = Colors.red;
  }

  //初始化状态
  init() {
    iconColor = Colors.white;
    icon = WitsIcon.duankai;
    _stop();
  }

  //开始
  _stateAnimation() {
    _controller?.forward();
    _changeController?.forward();
  }

  //停止
  _stop() {
    _controller?.stop(canceled: false);
    _controller?.reset();
    _changeController?.reverse();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _heartbeatController?.forward(); //页面显示启动心跳
    if (widget.immediatelyExecute) {
      Future.delayed(const Duration(milliseconds: 20), () {
        widget.onTop(this);
      });
    }
  }

  @override
  void dispose() {
    _controller?.stop();
    _controller?.dispose();

    _changeController?.stop();
    _changeController?.dispose();

    _heartbeatController?.stop();
    _heartbeatController?.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
        onTap: () {
          _onTop();
        },
        child: Center(
            child: CustomPaint(
                painter: _Unlock(
                    lineAnimation: _lineAnimation!,
                    changeLineAnimation: _changeAnimation!,
                    heartbeatAnimation: _heartbeatAnimation!,
                    icon: icon,
                    iconColor: iconColor,
                    heartColor: Theme.of(context).primaryColorLight),
                size: Size(widget.height, widget.width))));
  }
}

class _Unlock extends CustomPainter {
  final Animation<double> lineAnimation;
  final Animation<double> changeLineAnimation;
  final Animation<double> heartbeatAnimation;
  final IconData icon;
  final Color iconColor;
  final Color heartColor;
  final int number = 45; //小圆的总个数
  final double minCircleR = 3; //小圆的半径
  const _Unlock({
    required this.lineAnimation,
    required this.heartColor,
    required this.changeLineAnimation,
    required this.heartbeatAnimation,
    required this.icon,
    required this.iconColor,
  });

  @override
  void paint(Canvas canvas, Size size) {
    var rect = Offset.zero & size;
    if (changeLineAnimation.value == 0) {
      _canvasCircleDot(rect, canvas, size);
    } else {
      _canvasCircle(rect, canvas, size);
    }
    _canvasHeartbeat(rect, canvas, size);
    _canvasText(rect, canvas, size);
  }

  ///绘制圆圈
  _canvasCircle(Rect rect, Canvas canvas, Size size) {
    if (changeLineAnimation.value != 0) {
      //判断是否需要重复绘制圆
      var paint = Paint()
        ..isAntiAlias = true
        ..style = PaintingStyle.stroke
        ..strokeWidth = changeLineAnimation.value
        ..color = Colors.white;
      double r = rect.width / 2;
      var offset = Offset(rect.center.dx, rect.center.dy);
      canvas.drawCircle(offset, r, paint);
    }
    //判断是否需重新绘制圆点
    if (changeLineAnimation.value != minCircleR) {
      _canvasCircleDot(rect, canvas, size);
    }
  }

  ///绘制圆点
  _canvasCircleDot(Rect rect, Canvas canvas, Size size) {
    var paint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.stroke
      ..color = Colors.white;
    double r = rect.width / 2;
    paint.style = PaintingStyle.fill; //填充
    double minCircle = 360 / number; //每一个圆间隔的角度度
    double hudu; //每一个小圆直接的弧度
    Offset offset;
    for (int i = 0; i < number; i++) {
      hudu = (2 * Math.pi / 360) * minCircle * i + lineAnimation.value;
      offset = Offset(rect.center.dx + (r * Math.cos(hudu)),
          rect.center.dy + r * Math.sin(hudu));
      canvas.drawCircle(offset, minCircleR - changeLineAnimation.value, paint);
    }
  }

  ///绘制心跳
  _canvasHeartbeat(Rect rect, Canvas canvas, Size size) {
    var paint = Paint()
      ..isAntiAlias = true
      ..style = PaintingStyle.fill
      ..strokeWidth = 1
      ..color = heartColor;
    double r = rect.width / 2 - 15 - heartbeatAnimation.value;
    var offset = Offset(rect.center.dx, rect.center.dy);
    canvas.drawCircle(offset, r, paint);
  }

  ///绘制文字
  _canvasText(Rect rect, Canvas canvas, Size size) {
    //drawParagraph 也可以绘制文本
    double textSize =
        iconColor == Colors.red ? 40 - heartbeatAnimation.value : 40;
    TextPainter painter = TextPainter(
        text: TextSpan(
            text: String.fromCharCode(icon.codePoint),
            style: TextStyle(
                fontSize: textSize,
                color: iconColor,
                fontFamily: icon.fontFamily,
                package: icon.fontPackage)),
        //注意fontFamily 需要制定对应的icon fontFamily 才能显示
        textAlign: TextAlign.center,
        textDirection: TextDirection.ltr);
    Offset offset = Offset(0, rect.center.dy - textSize / 2);
    painter.layout(minWidth: size.width, maxWidth: size.width);
    painter.paint(canvas, offset);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
