import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/NetError.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterHintWidget.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterInputWidget.dart';
import 'package:witsystem_core/user/UserRemarks.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///用户的列表的item
class UserManageItemWidget extends StatefulWidget {
  final Map<String, dynamic> userInfo;
  final String deviceId;
  final Function(Map<String, dynamic>)? listRefurbishCall; //需要刷新整个列表的回调

  const UserManageItemWidget(
      {Key? key,
      required this.userInfo,
      required this.deviceId,
      this.listRefurbishCall})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _UserManageItemState();
  }
}

class _UserManageItemState extends State<UserManageItemWidget> {
  List<String> sheetItem = [
    S.current.device_deleteUser,
    S.current.device_freeze + "/" + S.current.device_unfreeze,
    S.current.device_remark,
    S.current.device_cancel
  ]; //底部弹框选项

  ///返回头像
  Widget _title() {
    return widget.userInfo["title_img"] == null
        ? Container(
            height: 60,
            width: 60,
            margin: const EdgeInsets.fromLTRB(5, 5, 10, 5),
            decoration: BoxDecoration(
                border: Border.all(color: Colors.black12),
                shape: BoxShape.circle),
            child: const Icon(WitsIcon.yonghu))
        : Container(
            height: 60,
            width: 60,
            margin: const EdgeInsets.fromLTRB(5, 5, 10, 5),
            decoration: BoxDecoration(
                border: Border.all(color: Colors.black12),
                shape: BoxShape.circle,
                image: DecorationImage(
                  image: NetworkImage(
                      "https://witsystem.top/user/find_title_img?fileName=${widget.userInfo["title_img"]}"),
                  fit: BoxFit.cover,
                )),
          );
  }

  ///点击item
  _clickItem() {
    WitsystemBottomSheet().show(
      context: context,
      sheetItem: sheetItem,
      onTap: (index) {
        _sheetClick(index);
      },
    );
  }

  ///弹出框回调
  _sheetClick(int index) {
    switch (index) {
      case 0:
        DialogCenterHint().show(
          context: context,
          title: S.current.device_deleteUserHintTitle,
          content: S.current
              .device_deleteUserHintContent(widget.userInfo["nickname"]),
          outSideDismissible: true,
          affirmText: S.of(context).device_delete,
          affirmTextColor: Colors.red,
          onPressed: (bool value) {
            affirmDelete(value);
          },
        );
        break;
      case 1:
        String state = widget.userInfo["freeze"]
            ? S.of(context).device_unfreeze
            : S.of(context).device_freeze;
        DialogCenterHint().show(
          context: context,
          title: S.current.device_deleteUserHintTitle,
          content:
              S.current.device_freezeHint(state, widget.userInfo["nickname"]),
          outSideDismissible: true,
          affirmText: state,
          affirmTextColor: Colors.red,
          onPressed: (bool value) {
            affirmFreeze(value);
          },
        );
        break;
      case 2:
        DialogCenterInput().show(
          context: context,
          title: S.current.device_remark,
          contentHint: S.current.device_remarksInfo,
          outSideDismissible: true,
          onPressed: (bool value, String text) {
            remarks(value, text);
          },
        );
        break;
    }
  }

  ///点击确认删除
  Future<void> affirmDelete(bool value) async {
    if (!value) return;
    try {
      EasyLoading.show(
          status: S.current.device_loading,
          maskType: EasyLoadingMaskType.clear);
      await DeviceManage.instance.manageDeleteDeviceUser(
          widget.deviceId, widget.userInfo["user_uuid"]);
      EasyLoading.dismiss();
      if (widget.listRefurbishCall != null) {
        widget.listRefurbishCall!(widget.userInfo);
      }
      EasyLoading.showSuccess(S.current.device_success);
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      EasyLoading.showError(NetError.error(int.parse(e.code)).error);
    }
  }

  ///点击冻结或者解冻回调
  Future<void> affirmFreeze(bool value) async {
    if (!value) return;
    try {
      EasyLoading.show(
          status: S.current.device_loading,
          maskType: EasyLoadingMaskType.clear);
      await DeviceManage.instance.freezeOrUnfreeze(widget.deviceId,
          widget.userInfo["user_uuid"], !widget.userInfo["freeze"]);
      widget.userInfo["freeze"] = !widget.userInfo["freeze"];
      if (widget.listRefurbishCall != null) {
        widget.listRefurbishCall!(widget.userInfo);
      }
      EasyLoading.showSuccess(S.current.device_success);
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      EasyLoading.showError(NetError.error(int.parse(e.code)).error);
    }
  }

  ///修改备注
  Future<void> remarks(bool value, String text) async {
    if (!value) return;
    EasyLoading.show(
        status: S.current.device_loading, maskType: EasyLoadingMaskType.clear);
    var updateRemarks =
        await UserRemarks().updateRemarks(widget.userInfo["user_id"], text);
    if (updateRemarks.code == NetError.succesCode) {
      EasyLoading.dismiss();
      widget.userInfo["nickname"] = text;
      setState(() {});
      EasyLoading.showSuccess(S.current.device_success);
    } else {
      EasyLoading.showError(updateRemarks.error);
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        _clickItem();
      },
      child: Container(
        height: 90,
        margin: const EdgeInsets.fromLTRB(0, 0, 0, 10),
        decoration: BoxDecoration(
          border: Border.all(
              color: widget.userInfo["freeze"] ? Colors.red : Colors.black12,
              width: 1),
          color: Colors.white,
          borderRadius: const BorderRadius.all(Radius.circular(10)),
          boxShadow: [
            BoxShadow(
                color: widget.userInfo["freeze"]
                    ? const Color(0xFFEF9A9A)
                    : Colors.black12,
                offset: const Offset(3, 3)),
          ],
        ),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Row(
              children: [
                _title(),
                Column(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(widget.userInfo["nickname"]),
                      Text(widget.userInfo["user_id"])
                    ]),
              ],
            ),
            Text.rich(TextSpan(style: const TextStyle(fontSize: 12), children: [
              TextSpan(
                  text: widget.userInfo["auth_type"] == 0
                      ? S.of(context).device_permanent
                      : widget.userInfo["end_date"]),
              const WidgetSpan(
                child: Icon(WitsIcon.youjiantou, size: 16),
              ),
            ]))
          ],
        ),
      ),
    );
  }
}
