
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/NetError.dart';
import 'package:witsystem_core/user/UserRemarks.dart';
import 'package:witsystem_device_manage/colors/ThemeColors.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/userManage/UserManageItem.dart';
import 'package:witsystem_device_manage/util/DeviceAuth.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///用户管理

class UserManageWidget extends StatefulWidget {
  final String deviceId;


  const UserManageWidget({Key? key, required this.deviceId}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return UserManageState();
  }
}

class UserManageState extends State<UserManageWidget> {
  String deviceName="";
  List<dynamic> list = [];

  @override
  void initState() {
    super.initState();
    _getDeviceName();
    init();
  }

  ///获得设备名称
  _getDeviceName() async {
    deviceName=  (await DeviceManage().device(widget.deviceId))!["deviceName"];
    setState(() {});
  }


  init() async {
    EasyLoading.show(
        status: S.current.device_loading, maskType: EasyLoadingMaskType.clear);
    try {
      List<dynamic> deviceUserList =
          await DeviceManage.instance.deviceUserList(widget.deviceId);
      list.clear();
      Map<int, List<dynamic>> cacheMap = {};
      Map? remarksInfo;
      for (var element in deviceUserList) {
        remarksInfo = await UserRemarks().getRemarksInfo(element["user_uuid"]);
        if (remarksInfo != null) element["nickname"] = remarksInfo["nickName"];
        if (cacheMap[element["auth_type"]] == null) {
          cacheMap[element["auth_type"]] = [
            {"itemType": DeviceAuth.intToStringAuth(element["auth_type"])}
          ];
        } //进行分类并且添加标题
        cacheMap[element["auth_type"]]?.add(element);
      }
      for (List<dynamic> element in cacheMap.values) {//分类好的数据重新组装
        list.addAll(element);
      }
      setState(() {});
      EasyLoading.dismiss();
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      EasyLoading.showError(NetError.error(int.parse(e.code)).error);
    }
  }



  @override
  void dispose() {
    super.dispose();
    EasyLoading.dismiss();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text(S.of(context).device_userManage+"($deviceName)",style: const TextStyle(fontSize: 16))),
      body: ListView.builder(
          itemCount: list.length,
          padding: const EdgeInsets.all(10),
          itemBuilder: (BuildContext context, int index) {
            return list[index]["itemType"] != null
                ? Container(
                    padding: const EdgeInsets.fromLTRB(0, 15, 0, 15),
                    child: Text(list[index]["itemType"],
                        style: const TextStyle(
                            color: ThemeColors.theme, fontSize: 22)))
                : UserManageItemWidget(
                    userInfo: list[index],
                    deviceId: widget.deviceId,
                    listRefurbishCall: (userInfo) async {
                      await init();
                    });
          }),
    );
  }
}
