import 'package:flutter/material.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/colors/ThemeColors.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/options/MoreOptions.dart';
import 'package:witsystem_device_manage/util/DeviceAuth.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';
import 'dart:convert' as convert;

import 'HomeListData.dart';

///首页模板

class HomeTempletWidget extends StatefulWidget {
  final List<HomeListData> list;

  final String deviceId;

  final String deviceName;

  final Widget child;

  final Function(HomeListData item, int index)? itemOnTap;

  const HomeTempletWidget(
      {Key? key,
      required this.deviceId,
      required this.list,
      required this.child,
      required this.deviceName,
      this.itemOnTap})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _HomeTempletState();
  }
}

class _HomeTempletState extends State<HomeTempletWidget> {
  int crossAxisCount = 4; //每一行的个数
  double childAspectRatio = 0; //item宽高比
  double containerHeight = 180; //底部item列表的高度
  Map<String, dynamic>? _device;

  double horizontalChildAspectRatio = 0; //横屏item宽高比
  double horizontalContainerHeight = 190; //横屏底部item列表的高度

  @override
  void initState() {
    super.initState();
    _filterItem();
    _getDeviceInfo();
  }

  ///筛选支持的功能
  _filterItem() async {
    List<HomeListData> list = [];
    int length = widget.list.length;
    for (int i = 0; i < length; i++) {
      var hardware = widget.list[i].hardware;
      if (hardware == null ||
          await DeviceManage.instance
              .isDeviceFunction(widget.deviceId, hardware)) {
        list.add(widget.list[i]);
      }
    }
    if (list.length < 9) {
      setState(() {});
      return; //判断如果小于7直接结束
    }
    var sublist = list.sublist(7); //分割出第8个以后的数据包含第8个
    list.removeRange(7, list.length); //删除第8个以后的数据包含第8个
    list.add(HomeListData(
        //给第8的位置添加更多选项
        icon: WitsIcon.gengduo,
        name: S.current.device_moreOptions,
        widget: MoreOptionsWidget(deviceId: widget.deviceId, list: sublist)));
    widget.list.clear();
    widget.list.addAll(list);
    setState(() {});
  }

  ///计算宽高比
  _reckonAspectRatio() {
    //屏幕宽度减去左右margin除以4就是每一个的宽度  containerHeight高度减去上padding除以2就是每一个的高度
    childAspectRatio =
        (((MediaQuery.of(context).size.width - 20) / crossAxisCount) /
            ((containerHeight - 10) / 2));

    //计算横屏状态的宽高比
    horizontalChildAspectRatio =
        (((MediaQuery.of(context).size.width / 2) / crossAxisCount) /
            ((horizontalContainerHeight - 10) / 2));
  }

  ///获得设备信息
  _getDeviceInfo() async {
    var _device = await DeviceManage().device(widget.deviceId);
    if (convert.json.encode(this._device) != convert.json.encode(_device)) {
      //判断如果数据发生变化进行刷新
      setState(() {
        this._device = _device;
      });
    }
  }

  ///每种设备中间部分item
  Widget _deviceCenterItem() {
    _getDeviceInfo();
    switch (_device == null ? "" : _device!["device"]["type"]) {
      case "Slock":
        return _slockCenterItem();

      case "Ammet":
        return _ammetCenterItem();

      case "Relay":
        return _relayCenterItem();

      default:
        return const SizedBox(width: 0, height: 0);
    }
  }

  ///门锁
  Widget _slockCenterItem() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        Text(S.of(context).device_battery("${_device!["device"]["battery"]}%"),
            style: const TextStyle(color: Colors.white, fontSize: 12)),
        Text(S.of(context).device_auth(DeviceAuth.enAuthToString(_device!["authType"])),
            style: const TextStyle(color: Colors.white, fontSize: 12))
      ],
    );
  }

  ///电表
  Widget _ammetCenterItem() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        Text(
            S.of(context).device_electricityPrice(
                _device!["device"]["devicesAmmet"]["electricity1"]),
            style: const TextStyle(color: Colors.white, fontSize: 12)),
        Text(
            S.of(context).device_auth(DeviceAuth.stringToStringEnAuth(
                _device!["device"]["authType"])),
            style: const TextStyle(color: Colors.white, fontSize: 12))
      ],
    );
  }

  ///中继
  Widget _relayCenterItem() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        Text(
            S.of(context).device_onLineState(_device!["device"]["onLine"]
                ? S.of(context).device_onLine
                : S.of(context).device_offLine),
            style: const TextStyle(color: Colors.white, fontSize: 12)),
        Text(
            S.of(context).device_auth(DeviceAuth.stringToStringEnAuth(
                _device!["authType"])),
            style: const TextStyle(color: Colors.white, fontSize: 12))
      ],
    );
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _reckonAspectRatio();
  }

  ///垂直布局
  Widget _verticalWidget() {
    return Flex(
      direction: Axis.vertical,
      children: [
        Expanded(
          flex: 1,
          child: widget.child,
        ),
        Container(
          margin: const EdgeInsets.fromLTRB(10, 0, 10, 0),
          padding: const EdgeInsets.all(6),
          decoration: const BoxDecoration(
            color: ThemeColors.themeItem,
            borderRadius: BorderRadius.all(Radius.circular(4.0)),
          ),
          child: _deviceCenterItem(),
        ),
        Container(
            height: containerHeight,
            padding: const EdgeInsets.fromLTRB(0, 5, 0, 5),
            margin: const EdgeInsets.all(10),
            decoration: const BoxDecoration(
              color: ThemeColors.themeItem,
              borderRadius: BorderRadius.all(Radius.circular(4.0)),
            ),
            child: GridView.builder(
                gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                    crossAxisCount: crossAxisCount,
                    crossAxisSpacing: 5,
                    mainAxisSpacing: 5,
                    childAspectRatio: childAspectRatio),
                itemCount: widget.list.length,
                itemBuilder: (context, index) {
                  return GestureDetector(
                    onTap: () {
                      if (widget.itemOnTap != null) {
                        widget.itemOnTap!(widget.list[index], index);
                      }
                    },
                    child: Container(
                      padding: const EdgeInsets.fromLTRB(5, 5, 5, 15),
                      color: ThemeColors.themeItem,
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          Icon(
                            widget.list[index].icon,
                            size: 30,
                            color: Colors.white,
                          ),
                          Text(
                            widget.list[index].name,
                            style: const TextStyle(
                                color: Colors.white, fontSize: 12),
                          )
                        ],
                      ),
                    ),
                  );
                })),
      ],
    );
  }

  ///水平布局
  Widget _horizontalWidget() {
    return Flex(
      direction: Axis.horizontal,
      children: [
        Expanded(
          flex: 1,
          child: widget.child,
        ),
        Expanded(
          flex: 1,
          child: Column(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Container(
                margin: const EdgeInsets.fromLTRB(10, 0, 10, 0),
                padding: const EdgeInsets.all(6),
                decoration: const BoxDecoration(
                  color: ThemeColors.themeItem,
                  borderRadius: BorderRadius.all(Radius.circular(4.0)),
                ),
                child: _deviceCenterItem(),
              ),
              Container(
                  height: horizontalContainerHeight,
                  padding: const EdgeInsets.fromLTRB(0, 5, 0, 5),
                  margin: const EdgeInsets.all(10),
                  decoration: const BoxDecoration(
                      color: ThemeColors.themeItem,
                    borderRadius: BorderRadius.all(Radius.circular(4.0)),
                  ),
                  child: GridView.builder(
                      gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                          crossAxisCount: crossAxisCount,
                          crossAxisSpacing: 5,
                          mainAxisSpacing: 5,
                          childAspectRatio: horizontalChildAspectRatio),
                      itemCount: widget.list.length,
                      itemBuilder: (context, index) {
                        return GestureDetector(
                          onTap: () {
                            if (widget.itemOnTap != null) {
                              widget.itemOnTap!(widget.list[index], index);
                            }
                          },
                          child: Container(
                            padding: const EdgeInsets.fromLTRB(5, 5, 5, 15),
                            color: ThemeColors.themeItem,
                            child: Column(
                              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                              mainAxisSize: MainAxisSize.min,
                              children: [
                                Icon(
                                  widget.list[index].icon,
                                  size: 30,
                                  color: Colors.white,
                                ),
                                Text(
                                  widget.list[index].name,
                                  style: const TextStyle(
                                      color: Colors.white, fontSize: 12),
                                )
                              ],
                            ),
                          ),
                        );
                      })),
            ],
          ),
        ),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.deviceName),
      ),
      body: Container(
        height: double.infinity,
        width: double.infinity,
        color: ThemeColors.theme,
        child: OrientationBuilder(
          builder: (BuildContext context, Orientation orientation) {
            return orientation == Orientation.portrait
                ? _verticalWidget()
                : _horizontalWidget();
          },
        ),
      ),
    );
  }
}
